#include "generators.h"

#ifndef UTILITY_STL_H
#define UTILITY_STL_H

namespace misc
{
	const std::size_t minval_radix = 2;
	const std::size_t maxval_radix = 7;

	void init(gen::ITEM*& a, \
		std::pair<std::size_t, std::size_t> range, std::size_t& count)
	{
		// Use STL Mersenne-Twister random values generator
		std::random_device rd; std::mt19937 gen(rd());
		std::uniform_int_distribution<> dist(range.first, range.second);

        if (count == 0)
			//count = dist(gen);
            count = 40;

		std::cout << std::setiosflags(std::ios::fixed) << std::setprecision(4)
			<< " (array size = " << count << ")..." << "\n";

		//gen::generate_objects(a, count);
	}

	std::size_t sorted(gen::ITEM* a, std::size_t size, std::size_t& position)
	{
		bool is_sorted = true;

   		tbb::parallel_for(tbb::blocked_range<std::size_t>(0, size - 1), \
            [&](const tbb::blocked_range<std::size_t>& r) {
			for (std::size_t index = r.begin(); index != r.end(); index++)
			{
                if (a[index].key == 0 || a[index].value == 0)	{
                    is_sorted = false; break;
                }

                else if ((a[index].key > a[index + 1].key) ||
                         (a[index].key == a[index + 1].key &&
                          a[index].value > a[index + 1].value))
                {
                    if (is_sorted == true)
                        position = index;

                    is_sorted = false;
                    
                    tbb::task::self().cancel_group_execution();
                }
            }
		});
        
		return is_sorted;
	}

	void print_out(gen::ITEM* a, std::size_t size)
	{
		// Print out the array
		for (std::size_t _Fwd = 0; _Fwd < size; _Fwd++)
			std::cout << "(" << a[_Fwd].key << " " << a[_Fwd].value << ") ";

		std::cout << "\n";

		std::cout << "\n";
	}
}

#endif // UTILITY_STL_H
